#!/bin/csh -f

# Copyright 2001 Bob Toxen.  All rights reserved.
# This program may be used under the terms of the
# GNU GENERAL PUBLIC LICENSE Version 2.
#
# Offered as is with no warranty of any kind.
# This script will overwrite up to 100 GB of
# free blocks on a file system to write over
# any possible confidential data.  It will
# work on Linux systems where there is a
# maximum file size of 2 GB.

# /usr/local/bin/fillup: fillup a file system to
# to obliterate any possibly confidential data
# that might be in the free blocks after the
# files containing it were removed.

# It expects a single argument which is
# a directory on the file system to be
# filled up.  When it is done it will
# invoke df and prompt the SysAdmin to
# verify that the df shows no free disk
# space and to hit return.

set fname="$1/junk$$"
if ( $#argv != 1 ) then
	echo "Usage: $0 directory"
	exit 1
endif
if ( ! -d $1 ) then
	echo "$1 is not a directory"
	exit 1
endif
if ( ! -o /bin/su ) then
	echo Not root
	exit 1
endif
unlimit -h filesize
unlimit filesize
df
# $i is quoted to protect against *
foreach i ( ${fname}{x,y}{1,2,3,4,5}{a,b,c,d,e} )
	echo Filling "$i"
	dd bs=1024k if=/dev/zero of="$i"
	df
end
echo "Verify that the last 'df' shows no free space"
echo -n '  then hit return to remove junk files: '
set z="$<"
foreach i ( ${fname}{x,y}{1,2,3,4,5}{a,b,c,d,e} )
	echo Removing "$i"
	/bin/rm -f "$i"
end
df
exit 0
