/*
 * Usage:
 *   tcpread [-f correct_file] [-b] [-s] [-o] [-d] [-l level]
 *     host path [seconds] [port]
 *
 * -f: Specify a file to compare against.  Status code indicates if match.
 *
 * -b: If not -f, write only body of page, not headers.  This will provide
 *     the actual HTTP, .GIF, .au, etc.
 *
 * -s: Silent: Only error codes and data.
 *
 * -o: Output contents to stdout even if -f.
 *
 * -d: Debug.
 *
 * Note: The path needs a leading slash.
 *
 * Copyright 2001 Bob Toxen.  All Rights Reserved.
 *
 * Purchasers of the book "Real World Linux Security --
 * Intrusion Prevention, Detection and Recovery" may copy this
 * script as needed to install and use on any system that they
 * administer.  Others may not copy or use it without obtaining
 * specific written permission by contacting the author at
 * book@cavu.com.
 *
 * Offered as is with no warranty of any kind.
 */

/*
 * $Log: tcpread.c,v $
 * Revision 1.3  2001/02/18 21:53:26  bob
 * Enhanced to provide headers that more servers will accept.
 *
 * Revision 1.2  2000/11/30 16:38:37  bob
 * Corrected a misspelling in an error message.
 *
 */

#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
#include <netdb.h>

#define	MAXCONN	20	/* Max seconds to attempt connection. */
#define MAXLINE 1024
#define	U	unsigned

int	sockfd;
int	socktype = SOCK_STREAM;
int	silent;
int	body;
int	out;
int	debug;
int	secs	= 60;
int	bytes;
char	*fname;
extern
int	errno;
char	*getblab=
"User-Agent: Mozilla/4.76[en] (X11; I; Linux 2.4 i586; Nav)\n"
"Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, image/png, */*\n"
"Accept-Encoding: gzip\n"
"Accept-Charset: iso-8859-1,*,utf-8\n";


static	char	*Sccsid = "@(#)$Header: /home/bob/book/it/web_add/RCS/tcpread.c,v 1.3 2001/02/18 21:53:26 bob Exp bob $";

tconn(U long inetAddr, int port)
{
	int	result	= 0;
	struct	sockaddr_in	serverAddr;
	static
	int	tries	= MAXCONN;


	serverAddr.sin_family = AF_INET;
	serverAddr.sin_addr.s_addr = inetAddr;
	serverAddr.sin_port = htons((U short) port);

	sockfd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (sockfd < 0) {
		perror("socket");
		exit(2);
	}
	if (debug >= 2)
		fprintf(stderr, "Connecting\n");
	for (;;) {
		result = connect(sockfd, (struct sockaddr *) &serverAddr,
		  sizeof serverAddr);
		if (result >= 0)
			break;
		if (--tries <= 0) {
			if (!silent) {
				if (result < 0)
					perror("connect failure");
				fprintf(stderr,
				  "Cannot connect after %d seconds\n",
				  MAXCONN);
			}
			exit(2);
		}
		sleep(1);
	}
	if (debug >= 2)
		fprintf(stderr, "Connected\n");
	return sockfd;
}

U long
gethost(char *name)
{
	struct	hostent	*host;
	struct	in_addr	*inetAddr;

	if (debug >= 2)
		fprintf(stderr, "Looking up host name\n");
	host = gethostbyname(name);
	if (!host || !host->h_addr_list[0]) {
		if (!silent)
			fprintf(stderr, "gethost: Could not find '%s'\n", name);
		return 0xFFFFFFFF;
	}
	return ((struct in_addr *) host->h_addr_list[0])->s_addr;
}

void
ring(int sig)
{
			/* Cannot use fprintf because we are interrupt. */
	if (!silent)
		write(2, "Timeout\n", strlen("Timeout\n"));
	exit(4);
}

main(int argc, char **argv)
{
	int	fd;
	int	port;
	U long	addr;
	FILE	*fp;
	FILE	*ffp	= 0;
	char	buf[1024];
	char	fbuf[1024];
	int	len;
	int	flen;
	char	*name;
	char	*msg;
	char	*level	= "HTTP/1.0";
	int	i;

	argc--;
	argv++;

	while (argc >= 1 && *argv[0] == '-' && strcmp(*argv, "-help")) {
		if (!strcmp(*argv, "-b"))
			body++;
		else if (!strcmp(*argv, "-s"))
			silent++;
		else if (!strcmp(*argv, "-o"))
			out++;
		else if (!strcmp(*argv, "-d"))
			debug++;
		else if (!strcmp(*argv, "-l") && argc >= 2) {
			argc--;
			argv++;
			level = *argv;
		}
		else if (!strcmp(*argv, "-f") && argc >= 2) {
			argc--;
			argv++;
			fname = *argv;
			ffp = fopen(fname, "r");
			if (!ffp) {
				if (!silent)
					perror(fname);
				exit(-2);
			}
		}
		argc--;
		argv++;
	}
	if (argc < 2 || argc == 1
	  && (!strcmp(*argv, "help") || !strcmp(*argv, "-help"))) {
		fprintf(stderr, "Usage: "
"tcpread [-f correct_file] [-b] [-s] [-o] [-d] [-l level] "
"host path [seconds] [port]\n");
		fprintf(stderr,
		  "  -f correct_file: file to compare web page to\n");
		fprintf(stderr,
		  "  -b: If not -f, write only body of page, not headers\n");
		fprintf(stderr,
		  "  -s: silent; return only status code and data\n");
		fprintf(stderr,
		  "  -o: Output contents to stdout even if -f\n");
		fprintf(stderr,
		  "  -d: debug; twice for more\n");
		fprintf(stderr,
		  "  -l level: Second argument to GET, e.g., \"HTTP/1.0\"\n");
		fprintf(stderr,
		  "  host: host name of web server\n");
		fprintf(stderr,
		  "  path: path of page on web server\n");
		fprintf(stderr,
		  "  seconds: seconds before timing out and returning error; "
		  "default: %d\n", secs);
		fprintf(stderr,
		  "  port: TCP/IP port to use; default: 80\n");
		fprintf(stderr,
		  "\nUnless -f is used, will output page to standard out\n");
		exit(2);
	}
	name = argv[0];
	msg  = argv[1];
	if (argc > 2) {
		secs = atoi(argv[2]);
	}
	if (secs > 0) {
		signal(SIGALRM, ring);
		alarm(secs);
	}
	if (argc > 3)
		port = atoi(argv[3]);
	else
		port = 80;

	addr = gethost(name);
	if (addr == 0xFFFFFFFF) {
		if (!silent)
			fprintf(stderr, "Could not convert name '%s'\n", name);
		exit(2);
	}
	if (debug)
		fprintf(stderr, "addr=%d.%d.%d.%d (derived from '%s')\n",
		  (addr >> 24) & 0xff,
		  (addr >> 16) & 0xff,
		  (addr >> 8)  & 0xff,
		  addr         & 0xff,
		  name);

	fd = tconn(addr, port);
	if (fd < 0)
		exit(2);
	fp = fdopen(fd, "rw");
	if (!fp) {
		if (!silent)
			perror("fdopen TCP");
		exit(2);
	}

	len = strlen(msg) + strlen(level);
	if (len >= sizeof buf - 20) {
		fprintf(stderr, "Message exceeds %d bytes\n", sizeof buf - 20);
		exit(3);
	}
	errno = 0;
	sprintf(buf, "GET %s %s\n%s\n", msg, level, getblab);
	if (debug >= 2)
		fprintf(stderr, "Writing request\n");
	if (write(fd, buf, strlen(buf)) != strlen(buf)) {
		perror("write");
		exit(-2);
	}
			/*
			 * Read first two lines,
			 * don't compare Date (2nd line).
			 */
	for (i=0; ; i++) {
		if (debug >= 2)
			fprintf(stderr, "Reading line %d\n", i+1);
		if (!fgets(buf, sizeof buf, fp)) {
			if (debug)
				fprintf(stderr, "Missing headers\n");
			if (!silent)
				fprintf(stderr, "DIFFERENT\n");
			exit(1);
		}
		if (debug >= 3)
			fprintf(stderr, "Read header#%d: '%s'\n", i+1, buf);
		len = strlen(buf);
		if (ffp) {
					/* Ignore actual date. */
			if (!fgets(fbuf, sizeof fbuf, ffp)
			  || i != 1 && strcmp(buf, fbuf)
			  || i == 1 && strncmp(buf, "Date: ", 6)) {
				if (debug)
					fprintf(stderr, "%s\n!=\n%s\n",
					  buf, fbuf);
				if (!silent)
					fprintf(stderr, "DIFFERENT\n");
				exit(1);
			}
		}
		if (!ffp || out) {
			if (!body && write(1, buf, len) != len) {
				if (!silent)
					perror("Write error");
				exit(2);
			}
		}
		bytes += len;
		if (!strcmp(buf, "\r\n") || !strcmp(buf, "\n"))
			break;	/* End of headers. */
	}
	if (debug >= 3)
		fprintf(stderr, "Past headers\n");

			/*
			 * Do binary compare on rest of data so that
			 * image files and the like work.
			 */
	for (;;) {
		if (debug >= 2)
			fprintf(stderr, "Reading from byte %d\n", bytes);
		len = fread(buf, 1, sizeof buf, fp);
		if (len > 0) {
			bytes += len;
			if (ffp) {
				flen = fread(fbuf, 1, len, ffp);
				if (flen != len || memcmp(buf, fbuf, len)) {
					if (debug) {
						if (flen != len)
							fprintf(stderr,
							  "TCP:%d bytes "
							  "f:%d\n",
							  len, flen);
						else
							fprintf(stderr,
							  "%.*s\n!=\n%.*s\n",
							  len, buf,
							  flen, fbuf);
					}
					if (!silent)
						fprintf(stderr, "DIFFERENT\n");
					exit(1);
				}
			}
			if (!ffp || out) {
				if (write(1, buf, len) != len) {
					if (!silent)
						perror("Write error");
					exit(2);
				}
			}
		} else if (!len) {
			if (ffp) {
				if (getc(ffp) != EOF) {
					if (debug)
						fprintf(stderr, "EOF on TCP\n");
					if (!silent)
						fprintf(stderr, "DIFFERENT\n");
					exit(1);
				}
				if (debug >= 2)
					fprintf(stderr, "All %d bytes match\n",
					  bytes);
			}
			exit(0);
		} else {
			perror("read");
			exit(2);
		}
	}
}
