/*
 * Copyright (c) Carnegie Mellon University 1994, 1995
 *
 * CERT(sm) Coordination Center
 * Software Engineering Institute
 * Carnegie Mellon University
 * Pittsburgh, PA 15213-3890
 * U.S.A.
 * CERT is a service mark of Carnegie Mellon University.
 *
 * This program is free software; you can distribute it and/or modify
 * it as long as you retain the Carnegie Mellon copyright statement.
 *
 * It can be obtained via anonymous FTP from 
 *	ftp://info.cert.org:/pub/tools/cpm.tar.Z
 *
 * Build:
 *   make cpm
 */


#include <stdio.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <net/if.h>

#define	MAXINTERFACES	16

main(int argc, char **argv)
{
	int	fd;
	int	numdevs;
	int	evildevs = 0;
	struct	ifreq	buf[MAXINTERFACES];
	struct	ifconf	ifc;
	char	str[256];
	int	i;
	int	done	= 0;

	argc--;
	argv++;
	while (argc > 0) {
	}

	ifc.ifc_len = sizeof buf;
	ifc.ifc_buf = (caddr_t) buf;

	fd = socket(AF_INET, SOCK_DGRAM, 0);
	if (fd < 0) {
		perror("cpm: socket");
		exit(-1);
	}
	if (ioctl(fd, SIOCGIFCONF, (char *) &ifc) < 0) {
		perror("cpm: ioctl");
		exit(-1);
	}

	numdevs = ifc.ifc_len/sizeof (struct ifreq);
	if (!numdevs) {
		fprintf(stderr, "cpm: No network numdevss found.\n");
		exit(-1);
	}
	fprintf(stderr, "%d network devices found:\n", numdevs);
	for (i=0; i<numdevs; i++) {
		fprintf(stderr, "%s: ", buf[i].ifr_name);
		if (!(ioctl(fd, SIOCGIFFLAGS, (char *) &buf[i]))) {
			if (buf[i].ifr_flags & IFF_PROMISC) {
				fprintf(stderr,
				  "*** IN PROMISCUOUS MODE ***\n");
				  evildevs++;
				  done = 1;
			} else
				fprintf(stderr, "Normal\n");
		} else {
			sprintf(str, "cpm: ioctl device %.100s",
			  buf[i].ifr_name);
			perror(str);
			done = 1;
		}
	}

	fprintf(stderr, "%d of them in promiscuous mode.\n", evildevs);

	close(fd);
	exit(evildevs);
}
